/*
 SPDX-License-Identifier: GPL-3.0-or-later
 myMPD (c) 2018-2022 Juergen Mang <mail@jcgames.de>
 https://github.com/jcorporation/mympd
*/

/**
 * This file contains central definitions and is included in each source file
 * as first include. Some defines are configured by cmake.
 */

#ifndef MYMPD_COMPILE_TIME_H
#define MYMPD_COMPILE_TIME_H

#include <limits.h>
#include <signal.h>

//build options
/* #undef MYMPD_DEBUG */
/* #undef MYMPD_ENABLE_LIBASAN */
#define MYMPD_EMBEDDED_ASSETS

//features
#define MYMPD_ENABLE_LIBID3TAG
#define MYMPD_ENABLE_FLAC
#define MYMPD_ENABLE_SSL
#define MYMPD_ENABLE_LUA
#define MYMPD_ENABLE_IPV6

//translation files
#define I18N_de_DE
#define I18N_en_US
/* #undef I18N_es_ES */
/* #undef I18N_es_VE */
/* #undef I18N_fi_FI */
#define I18N_fr_FR
#define I18N_it_IT
#define I18N_ja_JP
/* #undef I18N_ko_KR */
#define I18N_nl_NL
#define I18N_zh_CN

//myMPD version from cmake
#define MYMPD_VERSION "10.1.7"

//default paths from cmake
#define MYMPD_DOC_ROOT "/var/lib/mympd/empty"
#define MYMPD_WORK_DIR "/var/lib/mympd"
#define MYMPD_CACHE_DIR "/var/cache/mympd"
#define MYMPD_LUALIBS_PATH ""

//global variables
extern _Atomic int worker_threads;
//signal handler
extern sig_atomic_t s_signal_received;
//message queues
extern struct t_mympd_queue *web_server_queue;
extern struct t_mympd_queue *mympd_api_queue;
extern struct t_mympd_queue *mympd_script_queue;

//state file names
#define FILENAME_LAST_PLAYED "last_played_list"

//mpd partitions
#define MPD_PARTITION_DEFAULT "default"
#define MPD_PARTITION_ALL "!all!"

//default configuration
#define CFG_MYMPD_USER "mympd"
#define CFG_MYMPD_HTTP_HOST "0.0.0.0"
#define CFG_MYMPD_HTTP_PORT "80"
#define CFG_MYMPD_SSL true
#define CFG_MYMPD_SSL_PORT "443"
#define CFG_MYMPD_SSL_SAN ""
#define CFG_MYMPD_CUSTOM_CERT false
#define CFG_MYMPD_ACL ""
#define CFG_MYMPD_SCRIPTACL "+127.0.0.0/8"
#define CFG_MYMPD_LUALIBS "all"
#define CFG_MYMPD_LOGLEVEL 5 //notice
#define CFG_MYMPD_PIN_HASH ""
#define CFG_LOG_TO_SYSLOG false
#define CFG_COVERCACHE_KEEP_DAYS 31

//default partiton state settings
#define PARTITION_HIGHLIGHT_COLOR "#28a745"
#define PARTITION_MPD_STREAM_PORT 8000

//default mpd state settings
#define MYMPD_MPD_TAG_LIST "Album,AlbumArtist,Artist,Disc,Genre,Name,Title,Track"
#define MYMPD_MPD_KEEPALIVE true
#define MYMPD_MPD_TIMEOUT 30000 //milliseconds
#define MYMPD_MPD_HOST "localhost"
#define MYMPD_MPD_PORT 6600
#define MYMPD_MPD_PASS ""
#define MYMPD_MPD_BINARYLIMIT 8192 //bytes

//default mympd state settings
#define MYMPD_MUSIC_DIRECTORY "auto"
#define MYMPD_PLAYLIST_DIRECTORY "auto"
#define MYMPD_JUKEBOX_PLAYLIST "Database"
#define MYMPD_JUKEBOX_UNIQUE_TAG MPD_TAG_ARTIST
#define MYMPD_JUKEBOX_LAST_PLAYED 24 //hours
#define MYMPD_JUKEBOX_QUEUE_LENGTH 1
#define MYMPD_JUKEBOX_ENFORCE_UNIQUE true
#define MYMPD_COVERIMAGE_NAMES "cover,folder"
#define MYMPD_THUMBNAIL_NAMES "cover-sm,folder-sm"
#define MYMPD_TAG_LIST_SEARCH "Album,AlbumArtist,Artist,Genre,Title"
#define MYMPD_TAG_LIST_BROWSE "Album,AlbumArtist,Artist,Genre"
#define MYMPD_SMARTPLS_GENERATE_TAG_LIST "Genre"
#define MYMPD_LAST_PLAYED_COUNT 200
#define MYMPD_SMARTPLS true
#define MYMPD_SMARTPLS_SORT ""
#define MYMPD_SMARTPLS_PREFIX "myMPDsmart"
#define MYMPD_SMARTPLS_INTERVAL 14400 //seconds - 4 hours
#define MYMPD_BOOKLET_NAME "booklet.pdf"
#define MYMPD_AUTO_PLAY false
#define MYMPD_COLS_QUEUE_CURRENT "[\"Pos\",\"Title\",\"Artist\",\"Album\",\"Duration\"]"
#define MYMPD_COLS_SEARCH "[\"Title\",\"Artist\",\"Album\",\"Duration\"]"
#define MYMPD_COLS_BROWSE_DATABASE_DETAIL "[\"Track\",\"Title\",\"Duration\"]"
#define MYMPD_COLS_BROWSE_PLAYLISTS_DETAIL "[\"Pos\",\"Title\",\"Artist\",\"Album\",\"Duration\"]"
#define MYMPD_COLS_BROWSE_FILESYSTEM "[\"Pos\",\"Title\",\"Artist\",\"Album\",\"Duration\"]"
#define MYMPD_COLS_PLAYBACK "[\"Artist\",\"Album\"]"
#define MYMPD_COLS_QUEUE_LAST_PLAYED "[\"Pos\",\"Title\",\"Artist\",\"Album\",\"LastPlayed\"]"
#define MYMPD_COLS_QUEUE_JUKEBOX "[\"Pos\",\"Title\",\"Artist\",\"Album\"]"
#define MYMPD_COLS_BROWSE_RADIO_WEBRADIODB "[\"Name\",\"Country\",\"Language\",\"Genre\"]"
#define MYMPD_COLS_BROWSE_RADIO_RADIOBROWSER "[\"name\",\"country\",\"language\",\"tags\"]"
#define MYMPD_VOLUME_MIN 0 //prct
#define MYMPD_VOLUME_MAX 100 //prct
#define MYMPD_VOLUME_STEP 5 //prct
#define MYMPD_WEBUI_SETTINGS "{}"
#define MYMPD_LYRICS_USLT_EXT "txt"
#define MYMPD_LYRICS_SYLT_EXT "lrc"
#define MYMPD_LYRICS_VORBIS_USLT "LYRICS"
#define MYMPD_LYRICS_VORBIS_SYLT "SYNCEDLYRICS"
#define MYMPD_NAVBAR_ICONS "[{\"ligature\":\"home\",\"title\":\"Home\",\"options\":[\"Home\"]},"\
    "{\"ligature\":\"equalizer\",\"title\":\"Playback\",\"options\":[\"Playback\"]},"\
    "{\"ligature\":\"queue_music\",\"title\":\"Queue\",\"options\":[\"Queue\"]},"\
    "{\"ligature\":\"library_music\",\"title\":\"Browse\",\"options\":[\"Browse\"]},"\
    "{\"ligature\":\"search\",\"title\":\"Search\",\"options\":[\"Search\"]}]"

//http headers
#define EXTRA_HEADERS_CACHE "Cache-Control: max-age=604800\r\n"

#define EXTRA_HEADERS_MISC "X-Content-Type-Options: nosniff\r\n"

#define EXTRA_HEADERS_PERMISSIONS_POLICY "Permissions-Policy: accelerometer=(), ambient-light-sensor=(), autoplay=(self), "\
    "battery=(), camera=(), cross-origin-isolated=(), display-capture=(), document-domain=(), encrypted-media=(), "\
    "execution-while-not-rendered=(), execution-while-out-of-viewport=(), fullscreen=(self), geolocation=(), gyroscope=(), "\
    "keyboard-map=(), magnetometer=(), microphone=(), midi=(), navigation-override=(), payment=(), picture-in-picture=(), "\
    "publickey-credentials-get=(), screen-wake-lock=(), sync-xhr=(), usb=(), web-share=(), xr-spatial-tracking=()\r\n"

#define EXTRA_HEADERS_UNSAFE "Content-Security-Policy: default-src 'none'; "\
    "style-src 'self' 'unsafe-inline'; font-src 'self'; script-src 'self' 'unsafe-inline'; img-src * data:; "\
    "connect-src 'self' ws: wss:; manifest-src 'self'; "\
    "media-src *; frame-ancestors *; base-uri 'none';\r\n"\
    EXTRA_HEADERS_PERMISSIONS_POLICY\
    EXTRA_HEADERS_MISC\
    EXTRA_HEADERS_CACHE

#define EXTRA_HEADERS_SAFE "Content-Security-Policy: default-src 'none'; "\
    "style-src 'self'; font-src 'self'; script-src 'self'; img-src 'self' data:; "\
    "connect-src 'self' ws: wss:; manifest-src 'self'; "\
    "media-src *; frame-ancestors *; base-uri 'none'; "\
    "require-trusted-types-for 'script'\r\n"\
    EXTRA_HEADERS_PERMISSIONS_POLICY\
    EXTRA_HEADERS_MISC

#define EXTRA_HEADERS_SAFE_CACHE EXTRA_HEADERS_SAFE\
    EXTRA_HEADERS_CACHE

#define EXTRA_HEADERS_IMAGE EXTRA_HEADERS_MISC\
    EXTRA_HEADERS_CACHE

#define EXTRA_HEADER_CONTENT_ENCODING "Content-Encoding: gzip\r\n"

#define DIRECTORY_LISTING_CSS "h1{top:0;font-size:inherit;font-weight:inherit}address{bottom:0;font-style:normal}"\
    "h1,address{background-color:#343a40;color:#f8f9fa;padding:1rem;position:fixed;"\
    "box-sizing:border-box;width:100%;margin-top:0}body{margin:5rem 0;background-color:#f7f7f7;"\
    "color:#212529;font-family:sans-serif;font-size:1rem;font-weight:400;line-height:1.5}"\
    "table{border-collapse:collapse;margin:1rem}th{border-bottom:2px solid #dee2e6;"\
    "border-top:1px solid #dee2e6;text-align:left;padding:.3rem;font-family:inherit}"\
    "td{text-align:left;padding:.3rem;font-family:inherit;border-bottom:1px solid #dee2e6}"\
    "td:last-child{text-align:right}a,a:visited,a:active{color:#212529;text-decoration:none}"\
    "a:hover{text-decoration:underline}"

#define EXTRA_MIME_TYPES "avif=image/avif,flac=audio/flac,oga=audio/ogg,ogg=audio/ogg,"\
    "opus=audio/ogg,spx=audio/ogg,pem=application/x-x509-ca-cert,woff2=font/woff2,"\
    "m3u=audio/mpegurl,json=application/json"

//io defaults
#define OPEN_FLAGS_READ "re"
#define OPEN_FLAGS_READ_BIN "rbe"
#define OPEN_FLAGS_WRITE "we"

//log level
#define LOGLEVEL_MIN 0
#define LOGLEVEL_MAX 7

//certificates
#define CA_LIFETIME 3650 //days
#define CA_LIFETIME_MIN 365 //days
#define CA_KEY_LENGTH 4096 //bytes
#define CERT_LIFETIME 365 //days
#define CERT_LIFETIME_MIN 30 //days
#define CERT_KEY_LENGTH 2048 //bytes

//http limits
#define HTTP_CONNECTIONS_MAX 100
#define URI_LENGTH_MAX 1000
#define BODY_SIZE_MAX 8192 //bytes

//session limits
#define HTTP_SESSIONS_MAX 10
#define HTTP_SESSION_TIMEOUT 1800 //seconds

//content limits
#define NAME_LEN_MAX 200
#define CONTENT_LEN_MAX 3000
#define EXPRESSION_LEN_MAX 1000
#define MPD_OUTPUT_ID_MAX 40
#define COLS_MAX 20

//mpd limits
#define MPD_PORT_MIN 1024
#define MPD_PORT_MAX 65535
#define MPD_TIMEOUT_MIN 10000 //ms
#define MPD_TIMEOUT_MAX 1000000 //ms
#define MPD_RESULTS_MIN 1
#define MPD_RESULTS_MAX 4000
#define MPD_COMMANDS_MAX 4000 //max number of commands for mpd command lists
#define MPD_PLAYLIST_LENGTH_MAX INT_MAX //max mpd queue or playlist length
#define MPD_BINARY_SIZE_MIN 4096 //4 kb mpd default
#define MPD_BINARY_SIZE_MAX 5242880 //5 MB
#define MPD_QUEUE_PRIO_MAX 255
#define MPD_CROSSFADE_MAX 100
#define MPD_CONNECTION_MAX 100

//limits for json parsing
#define JSONRPC_INT_MIN INT_MIN
#define JSONRPC_INT_MAX INT_MAX
#define JSONRPC_LONG_MIN INT_MIN
#define JSONRPC_LONG_MAX INT_MAX
#define JSONRPC_LLONG_MIN LLONG_MIN
#define JSONRPC_LLONG_MAX LLONG_MAX
#define JSONRPC_UINT_MAX INT_MAX
#define JSONRPC_STR_MAX 3000
#define JSONRPC_KEY_MAX 50

//some other limits
#define TIMER_INTERVAL_MIN 5 //seconds
#define TIMER_INTERVAL_MAX 7257600 //12 weeks
#define COVERCACHE_AGE_MIN 0 //days
#define COVERCACHE_AGE_MAX 365 //days
#define COVERCACHE_CLEANUP_OFFSET 60 //seconds
#define COVERCACHE_CLEANUP_INTERVAL 86400 //seconds
#define VOLUME_MIN 0 //prct
#define VOLUME_MAX 100 //prct
#define VOLUME_STEP_MIN 1 //prct
#define VOLUME_STEP_MAX 25 //prct
#define JUKEBOX_MODE_MIN 0
#define JUKEBOX_MODE_MAX 2
#define SCROBBLE_TIME_MIN 10 //minimum song length in seconds for the scrobble event
#define SCROBBLE_TIME_MAX 240 //maximum elapsed seconds before scrobble event occurs
#define SCROBBLE_TIME_TOTAL 480

//limits for lists
#define LIST_HOME_ICONS_MAX 99
#define LIST_TRIGGER_MAX 99
#define LIST_TIMER_MAX 99
#define USER_TIMER_ID_START 100
#define USER_TIMER_ID_MIN 101
#define USER_TIMER_ID_MAX 200
#define JUKEBOX_QUEUE_MIN 0
#define JUKEBOX_QUEUE_MAX 999
#define JUKEBOX_LAST_PLAYED_MIN 0
#define JUKEBOX_LAST_PLAYED_MAX 5000
#define SCRIPT_ARGUMENTS_MAX 20

//filesystem limits
#define FILENAME_LEN_MAX 200
#define FILEPATH_LEN_MAX 1000

//file size limits
#define LINE_LENGTH_MAX 2000
#define LYRICS_SIZE_MAX 10000 //bytes
#define SCRIPTS_SIZE_MAX 10000 //bytes
#define SMARTPLS_SIZE_MAX 2000 //bytes
#define WEBRADIODB_SIZE_MAX 1048576 //bytes, 1 MB

//limits for stickers
#define STICKER_PLAY_COUNT_MAX INT_MAX / 2
#define STICKER_SKIP_COUNT_MAX INT_MAX / 2

//cloud api hosts
#define RADIOBROWSER_HOST "all.api.radio-browser.info"
#define WEBRADIODB_HOST "jcorporation.github.io"

#endif
